@@ -10,9 +10,9 @@ from django.conf import settings
 
 # urlparse in python3 has been renamed to urllib.parse
 try:
-    from urlparse import urlparse, parse_qs, urlunparse
+    from urlparse import urlparse, parse_qs, parse_qsl, urlunparse
 except ImportError:
-    from urllib.parse import urlparse, parse_qs, urlunparse
+    from urllib.parse import urlparse, parse_qs, parse_qsl, urlunparse
 
 try:
     from urllib import urlencode, unquote_plus
@@ -30,3 +30,16 @@ try:
 except ImportError:
     from django.contrib.auth.models import User
     get_user_model = lambda: User
+
+# Django's new application loading system
+try:
+    from django.apps import apps
+    get_model = apps.get_model
+except ImportError:
+    from django.db.models import get_model
+
+# Django 1.5 add the support of context variables for the url template tag
+if django.VERSION >= (1, 5):
+    from django.template.defaulttags import url
+else:
+    from django.templatetags.future import url
