@@ -1,6 +1,6 @@
 class OAuthToolkitError(Exception):
     """
-
+    Base class for exceptions
     """
     def __init__(self, error=None, redirect_uri=None, *args, **kwargs):
         super(OAuthToolkitError, self).__init__(*args, **kwargs)
@@ -12,4 +12,6 @@ class OAuthToolkitError(Exception):
 
 class FatalClientError(OAuthToolkitError):
     """
+    Class for critical errors
     """
+    pass
