@@ -59,3 +59,28 @@ class TokenHasReadWriteScope(TokenHasScope):
             read_write_scope = oauth2_settings.WRITE_SCOPE
 
         return required_scopes + [read_write_scope]
+
+
+class TokenHasResourceScope(TokenHasScope):
+    """
+    The request is authenticated as a user and the token used has the right scope
+    """
+
+    def get_scopes(self, request, view):
+        try:
+            view_scopes = (
+                super(TokenHasResourceScope, self).get_scopes(request, view)
+            )
+        except ImproperlyConfigured:
+            view_scopes = []
+
+        if request.method.upper() in SAFE_HTTP_METHODS:
+            scope_type = oauth2_settings.READ_SCOPE
+        else:
+            scope_type = oauth2_settings.WRITE_SCOPE
+
+        required_scopes = [
+            '{0}:{1}'.format(scope, scope_type) for scope in view_scopes
+        ]
+
+        return required_scopes
