@@ -1,20 +1,17 @@
 from __future__ import unicode_literals
 
+from datetime import timedelta
+
 from django.core.urlresolvers import reverse
-from django.db import models
+from django.db import models, transaction
 from django.utils import timezone
-try:
-    # Django's new application loading system
-    from django.apps import apps
-    get_model = apps.get_model
-except ImportError:
-    from django.db.models import get_model
+
 from django.utils.translation import ugettext_lazy as _
 from django.utils.encoding import python_2_unicode_compatible
 from django.core.exceptions import ImproperlyConfigured
 
 from .settings import oauth2_settings
-from .compat import AUTH_USER_MODEL
+from .compat import AUTH_USER_MODEL, parse_qsl, urlparse, get_model
 from .generators import generate_client_secret, generate_client_id
 from .validators import validate_uris
 
@@ -60,7 +57,7 @@ class AbstractApplication(models.Model):
 
     client_id = models.CharField(max_length=100, unique=True,
                                  default=generate_client_id, db_index=True)
-    user = models.ForeignKey(AUTH_USER_MODEL)
+    user = models.ForeignKey(AUTH_USER_MODEL, related_name="%(app_label)s_%(class)s")
     help_text = _("Allowed URIs list, space separated")
     redirect_uris = models.TextField(help_text=help_text,
                                      validators=[validate_uris], blank=True)
@@ -70,6 +67,7 @@ class AbstractApplication(models.Model):
     client_secret = models.CharField(max_length=255, blank=True,
                                      default=generate_client_secret, db_index=True)
     name = models.CharField(max_length=255, blank=True)
+    skip_authorization = models.BooleanField(default=False)
 
     class Meta:
         abstract = True
@@ -93,7 +91,21 @@ class AbstractApplication(models.Model):
 
         :param uri: Url to check
         """
-        return uri in self.redirect_uris.split()
+        for allowed_uri in self.redirect_uris.split():
+            parsed_allowed_uri = urlparse(allowed_uri)
+            parsed_uri = urlparse(uri)
+
+            if (parsed_allowed_uri.scheme == parsed_uri.scheme and
+                    parsed_allowed_uri.netloc == parsed_uri.netloc and
+                    parsed_allowed_uri.path == parsed_uri.path):
+
+                aqs_set = set(parse_qsl(parsed_allowed_uri.query))
+                uqs_set = set(parse_qsl(parsed_uri.query))
+
+                if aqs_set.issubset(uqs_set):
+                    return True
+
+        return False
 
     def clean(self):
         from django.core.exceptions import ValidationError
@@ -145,6 +157,9 @@ class Grant(models.Model):
         """
         Check token expiration with timezone awareness
         """
+        if not self.expires:
+            return True
+
         return timezone.now() >= self.expires
 
     def redirect_uri_allowed(self, uri):
@@ -165,11 +180,10 @@ class AccessToken(models.Model):
     * :attr:`user` The Django user representing resources' owner
     * :attr:`token` Access token
     * :attr:`application` Application instance
-    * :attr:`expires` Expire time in seconds, defaults to
-                      :data:`settings.ACCESS_TOKEN_EXPIRE_SECONDS`
+    * :attr:`expires` Date and time of token expiration, in DateTime format
     * :attr:`scope` Allowed scopes
     """
-    user = models.ForeignKey(AUTH_USER_MODEL)
+    user = models.ForeignKey(AUTH_USER_MODEL, blank=True, null=True)
     token = models.CharField(max_length=255, db_index=True)
     application = models.ForeignKey(oauth2_settings.APPLICATION_MODEL)
     expires = models.DateTimeField()
@@ -187,6 +201,9 @@ class AccessToken(models.Model):
         """
         Check token expiration with timezone awareness
         """
+        if not self.expires:
+            return True
+
         return timezone.now() >= self.expires
 
     def allow_scopes(self, scopes):
@@ -203,6 +220,20 @@ class AccessToken(models.Model):
 
         return resource_scopes.issubset(provided_scopes)
 
+    def revoke(self):
+        """
+        Convenience method to uniform tokens' interface, for now
+        simply remove this token from the database in order to revoke it.
+        """
+        self.delete()
+
+    @property
+    def scopes(self):
+        """
+        Returns a dictionary of allowed scope names (as keys) with their descriptions (as values)
+        """
+        return {name: desc for name, desc in oauth2_settings.SCOPES.items() if name in self.scope.split()}
+
     def __str__(self):
         return self.token
 
@@ -227,6 +258,13 @@ class RefreshToken(models.Model):
     access_token = models.OneToOneField(AccessToken,
                                         related_name='refresh_token')
 
+    def revoke(self):
+        """
+        Delete this refresh token along with related access token
+        """
+        AccessToken.objects.get(id=self.access_token.id).revoke()
+        self.delete()
+
     def __str__(self):
         return self.token
 
@@ -243,3 +281,24 @@ def get_application_model():
         e = "APPLICATION_MODEL refers to model {0} that has not been installed"
         raise ImproperlyConfigured(e.format(oauth2_settings.APPLICATION_MODEL))
     return app_model
+
+
+def clear_expired():
+    now = timezone.now()
+    refresh_expire_at = None
+
+    REFRESH_TOKEN_EXPIRE_SECONDS = oauth2_settings.REFRESH_TOKEN_EXPIRE_SECONDS
+    if REFRESH_TOKEN_EXPIRE_SECONDS:
+        if not isinstance(REFRESH_TOKEN_EXPIRE_SECONDS, timedelta):
+            try:
+                REFRESH_TOKEN_EXPIRE_SECONDS = timedelta(seconds=REFRESH_TOKEN_EXPIRE_SECONDS)
+            except TypeError:
+                e = "REFRESH_TOKEN_EXPIRE_SECONDS must be either a timedelta or seconds"
+                raise ImproperlyConfigured(e)
+        refresh_expire_at = now - REFRESH_TOKEN_EXPIRE_SECONDS
+
+    with transaction.atomic():
+        if refresh_expire_at:
+            RefreshToken.objects.filter(access_token__expires__lt=refresh_expire_at).delete()
+        AccessToken.objects.filter(refresh_token__isnull=True, expires__lt=now).delete()
+        Grant.objects.filter(expires__lt=now).delete()
