@@ -1,5 +1,7 @@
 from __future__ import unicode_literals
 
+import json
+
 from oauthlib import oauth2
 from oauthlib.common import urlencode, urlencoded, quote
 
@@ -16,7 +18,7 @@ class OAuthLibCore(object):
         """
         :params server: An instance of oauthlib.oauth2.Server class
         """
-        self.server = server or oauth2.Server(oauth2_settings.OAUTH2_VALIDATOR_CLASS())
+        self.server = server or oauth2_settings.OAUTH2_SERVER_CLASS(oauth2_settings.OAUTH2_VALIDATOR_CLASS())
 
     def _get_escaped_full_path(self, request):
         """
@@ -26,17 +28,38 @@ class OAuthLibCore(object):
         parsed = list(urlparse(request.get_full_path()))
         unsafe = set(c for c in parsed[4]).difference(urlencoded)
         for c in unsafe:
-            parsed[4] = parsed[4].replace(c, quote(c, safe=''))
+            parsed[4] = parsed[4].replace(c, quote(c, safe=b''))
 
         return urlunparse(parsed)
 
+    def _get_extra_credentials(self, request):
+        """
+        Produce extra credentials for token response. This dictionary will be
+        merged with the response.
+        See also: `oauthlib.oauth2.rfc6749.TokenEndpoint.create_token_response`
+
+        :param request: The current django.http.HttpRequest object
+        :return: dictionary of extra credentials or None (default)
+        """
+        return None
+
     def _extract_params(self, request):
         """
         Extract parameters from the Django request object. Such parameters will then be passed to
-        OAuthLib to build its own Request object
+        OAuthLib to build its own Request object. The body should be encoded using OAuthLib urlencoded
         """
         uri = self._get_escaped_full_path(request)
         http_method = request.method
+        headers = self.extract_headers(request)
+        body = urlencode(self.extract_body(request))
+        return uri, http_method, body, headers
+
+    def extract_headers(self, request):
+        """
+        Extracts headers from the Django request object
+        :param request: The current django.http.HttpRequest object
+        :return: a dictionary with OAuthLib needed headers
+        """
         headers = request.META.copy()
         if 'wsgi.input' in headers:
             del headers['wsgi.input']
@@ -44,8 +67,16 @@ class OAuthLibCore(object):
             del headers['wsgi.errors']
         if 'HTTP_AUTHORIZATION' in headers:
             headers['Authorization'] = headers['HTTP_AUTHORIZATION']
-        body = urlencode(request.POST.items())
-        return uri, http_method, body, headers
+
+        return headers
+
+    def extract_body(self, request):
+        """
+        Extracts the POST body from the Django request object
+        :param request: The current django.http.HttpRequest object
+        :return: provided POST parameters
+        """
+        return request.POST.items()
 
     def validate_authorization_request(self, request):
         """
@@ -101,9 +132,25 @@ class OAuthLibCore(object):
         :param request: The current django.http.HttpRequest object
         """
         uri, http_method, body, headers = self._extract_params(request)
+        extra_credentials = self._get_extra_credentials(request)
 
         headers, body, status = self.server.create_token_response(uri, http_method, body,
-                                                                  headers)
+                                                                  headers, extra_credentials)
+        uri = headers.get("Location", None)
+
+        return uri, headers, body, status
+
+    def create_revocation_response(self, request):
+        """
+        A wrapper method that calls create_revocation_response on a
+        `server_class` instance.
+
+        :param request: The current django.http.HttpRequest object
+        """
+        uri, http_method, body, headers = self._extract_params(request)
+
+        headers, body, status = self.server.create_revocation_response(
+            uri, http_method, body, headers)
         uri = headers.get("Location", None)
 
         return uri, headers, body, status
@@ -121,12 +168,29 @@ class OAuthLibCore(object):
         return valid, r
 
 
+class JSONOAuthLibCore(OAuthLibCore):
+    """
+    Extends the default OAuthLibCore to parse correctly requests with application/json Content-Type
+    """
+    def extract_body(self, request):
+        """
+        Extracts the JSON body from the Django request object
+        :param request: The current django.http.HttpRequest object
+        :return: provided POST parameters "urlencodable"
+        """
+        try:
+            body = json.loads(request.body.decode('utf-8')).items()
+        except ValueError:
+            body = ""
+
+        return body
+
+
 def get_oauthlib_core():
     """
     Utility function that take a request and returns an instance of
     `oauth2_provider.backends.OAuthLibCore`
     """
-    from oauthlib.oauth2 import Server
-
-    server = Server(oauth2_settings.OAUTH2_VALIDATOR_CLASS())
-    return OAuthLibCore(server)
+    validator = oauth2_settings.OAUTH2_VALIDATOR_CLASS()
+    server = oauth2_settings.OAUTH2_SERVER_CLASS(validator)
+    return oauth2_settings.OAUTH2_BACKEND_CLASS(server)
